<?php

/**
 * Dynamic Generation of Map Image
 * Loads the Background Image and fills in Icons for the Elements, depending
 * on their states.
 * 
 * @global string $elements
 * @global type $maps
 * @global type $states
 * @param type $mapname Name of Background Image for Map
 * @return type 
 */
function GenerateMapImage($mapname) {
    global $elements, $maps, $states;

    /**
     * Array of states of all defined Elements in fhem
     * Parses the xml output of the fhem command "xmllist"
     */
    $states = CreateStateList();
    /**
     * Filename for Background Image
     */
    $bg_filename = $maps[$mapname]["img"];

    /**
     * Image Ressource created from Background Image
     */
    $mapimg = ImageCreateFromPng($bg_filename);
    /**
     * Background Color to be used to draw to Image
     */
    $bg_color = ImageColorAllocate($mapimg, 255, 255, 255);
    /**
     * Foreground Color to be used to draw to Image
     */
    $fg_color = ImageColorAllocate($mapimg, 255, 0, 0);

    /*
     * Loop through all Elements to be displayed on the specified Map
     * Get the State of the Element
     * Insert an Icon depending on the State on the Background Image
     */
    foreach ($elements as $key => $element) {
        //Check if current Element is to be displayed in current Map
        if ($element["map"] == $mapname) {
            // Read the current state of the Element
            $state[$key] = GetState($key);
            // Get the Type of the Element as defined in symbols.php
            $typ = strtolower($element["typ"]);
            // Get the Icon Prefix as defined in symbols.php
            $imgpre = strtolower($element["imgpre"]);
            // Generate Icon depending on Type of Element
            switch ($typ) {
                case "sd":  // For Powerplugs...
                case "la":  // And Lamps, use the same procedure:
                    // Read the Icon depensing on the State ("on" or "off")
                    $element_img = ImageCreateFromPng("icons/" . $imgpre . "_" . (($state[$key] == "on") ? "on" : "off") . ".png");
                    break;
                case "tf":  // For Weather Sensor:
                    $tfarr = explode(" ", $state[$key]);    // GetState delivers two values in a string, separated by Whitespace
                    $text = $tfarr[0] . chr(176) . "C\n" . $tfarr[1] . "%";   // Text to be Displayed: "xx.x°C\nxx.x%"
                    $element_img = CreateSymbolFromText($text); // Generate Image Ressource from text
                    break;
                case "ht":  // For Radiator Thermostat Head:
                    $text = $state[$key] . "%"; // Display the actual Setting in %
                    $element_img = CreateSymbolFromText($text); // Generate Image Ressource from text
                    break;
                case "fht": // For Room Controller:
                    $text = $state[$key];   // Simply Display the State; GetState does the Formatting already
                    $element_img = CreateSymbolFromText($text); // Generate Image Ressource from text
                    break;
                case "pid": // For PID Controller:
                    $text = $state[$key];   // Simply Display the State; GetState does the Formatting already
                    $element_img = CreateSymbolFromText($text); // Generate Image Ressource from text
                    break;
                default:    // If undefined, create a Default Icon:
                    $text = "???";  // Display "???"
                    $element_img = CreateSymbolFromText($text); // Generate Image Ressource from text
                    break;
            }
            /*
             * Calculate the Coordinates for the Image:
             * Left and Top from symbols.php, Width and Height from Image Ressource $element_img
             */
            $l = $element["left"];
            $t = $element["top"];
            $w = imagesx($element_img);
            $h = imagesy($element_img);
            // Create the coords-String usable by AREA-Tags and save to $elements
            $elements[$key]["coords"] = $l . "," . $t . "," . ($l + $w) . "," . ($t + $h);
            // Draw Icon on Background Image
            ImageCopy($mapimg, $element_img, $l, $t, 0, 0, $w, $h);
        }
    }
    // Return the created Image
    return $mapimg;
}

/**
 * Generate a List of available Maps
 * Shows Links for every defined Map (see symbols.php)
 * 
 * @global type $maps
 * @global type $me 
 */
function ListMaps() {
    global $maps, $me;
    // Loop through all Maps
    foreach ($maps as $key => $map) {
        // Generate a line "<a href='/me/self?map=newmap'>newmapcomment</a>" for every Map
        $line = "<a href=\"" . $me . '?map=' . $key . "\">" . $map["comment"] . "</a><br>\n";
        echo $line;
    }
}

/**
 * Generate the Area Tags for the Map
 * Loops through all Elements and prints "<area..." tags for each displayed Element
 * 
 * @global string $elements
 * @global type $me 
 * @param type $mapname Name of Map to be used
 */
function GenerateMap($mapname) {
    global $elements, $me;
    foreach ($elements as $key => $element) {
        if ($element["map"] == $mapname) {
            $typ = strtolower($element["typ"]);
            $line = "<area id=\"" . $key . "\"" .
                    " shape=\"rect\" coords=\"" . $element['coords'] . "\"" .
                    " href=\"#" . $key . "\"" .
                    " onClick=\"ShowCmdBox(event, 'cmdbox', '" . $me . "', '" . $key . "', '" . $typ . "')\"" .
                    " alt=\"" . $key . "\">\n";
            echo $line;
        }
    }
}

/**
 * Generate a Command String from Template
 * Tages $fhemcmdtmpl from config.php and replaces the Substring $fhemcmdtag by $cmd
 *
 * @global type $fhemcmdtmpl Template as defined in config.php
 * @global type $fhemcmdtag Tag to Replace in Template
 * @param type $cmd Commandstring to replace the Tag with
 * @return type Command String to be executed by exec()
 */
function GenerateCmdString($cmd){
    global $fhemcmdtmpl, $fhemcmdtag;
    $retv = str_replace($fhemcmdtag, $cmd, $fhemcmdtmpl);
    return $retv;
}

/**
 * Generate the State List
 * Calls fhem to generate a xml output file
 * Parses this file and generates a named Array with all States
 * Builds a Tree-like Array:
 * ARRAY
 *   |
 *   +-- Elementname -+- Attribute1 -> value
 *   |                |
 *   |                +- Attribute2 -> value
 *   +-- NextElement -+- Attribute1 -> value
 *   .                .
 *
 * @global type $fhemxmllistopt Special Options only needed for XMLList Creating
 * @return type Returns an array of all States
 */
function CreateStateList() {
    global $fhemxmllistopt;
    // Call fhem with Command "xmllist"
    $syscall = GenerateCmdString("xmllist;quit") . " " . $fhemxmllistopt . " > fhem.xml";
    exec($syscall);
    // Load the newly generated xml file using simlexml Library
    $xmldata = simplexml_load_file("fhem.xml");
    // Run through the xml data and generate the array
    foreach ($xmldata->children() as $first) {
        foreach ($first->children() as $second) {
            $name = (string) $second["name"];
            $fhemdata[$name] = array();
            foreach ($second->children() as $third) {
                $key = strtolower((string) $third["key"]);
                $value = (string) $third["value"];
                $fhemdata[$name][$key] = $value;
            }
        }
    }
    // Return the Array
    return $fhemdata;
}

/**
 * Creates an Image Ressource containing the Text passed
 * 
 * @global type $fontsize
 * @global type $fontfile
 * @param type $text
 * @return type Image Ressource of Text
 */
function CreateSymbolFromText($text) {
    global $fontsize, $fontfile;
    $textsize = imageftbbox($fontsize, 0, $fontfile, $text);
    $tw = $textsize[2] - $textsize[0];
    $th = $textsize[1] - $textsize[5];
    $tofft = $textsize[5];
    $element_img = ImageCreate($tw, $th);
    ImageFill($element_img, 0, 0, ImageColorAllocate($element_img, 255, 255, 255));
    ImageFTText($element_img, $fontsize, 0, 0, -$tofft, ImageColorAllocate($element_img, 0, 0, 0), $fontfile, $text);
    return $element_img;
}

/**
 * Get the State of a named Element
 * Returns the current State of element $name
 * This Function is a wrapper for type-specific GetState-Functions
 * 
 * @global string $elements
 * @global type $states
 * @param type $name Name of the Element which state to return
 * @return string Current State of Element $name
 */
function GetState($name) {
    global $elements, $states;
    /**
     * Type of Element as defined in symbols.php
     */
    $typ = $elements[$name]["typ"];
    switch (strtolower($typ)) {
        case "sd":  // Switchable Plug or
        case "la":  // Lamp (not dimmer)
            $res = GetStateSDLA($name);
            break;
        case "tf":  // Weather Sensor (Temperature and Humidity)
            $res = GetStateTF($name);
            break;
        case "ht":  // Radiator Thermostatic Head
            $res = GetStateHT($name);
            break;
        case "fht": // Room Thermostate
            $res = GetStateFHT($name);
            break;
        case "pid": // PID Controller
            $res = GetStatePID($name);
            break;
        default:    // If not declared, return "???"
            $res = "??";
            break;
    }
    // Return current State
    return $res;
}

/**
 * Set the State of a named Element
 * Sets the Elemenet $name to State $state
 * This Function is a wrapper for type-specific SetState-Functions
 * 
 * @global string $elements
 * @param type $name Name of the Element to set the State
 * @param type $state State to set the Element to
 */
function SetState($name, $state) {
    global $elements;
    /**
     * Type of Element as defined in symbols.php
     */
    $typ = $elements[$name]["typ"];
    switch (strtolower($typ)) {
        case "sd":  // Switchable Plug
            SetStateSD($name, $state);
            break;
        case "la":  // Lamp
            SetStateLA($name, $state);
            break;
        case "ht":  // Radiator Thermostatic Head
            SetStateHT($name, $state);
            break;
        case "fht": // Room Thermostate
            SetStateFHT($name, $state);
            break;
        case "pid": // PID Controller
            SetStatePID($name, $state);
            break;
        default:    // Undefined: Do nothing
            break;
    }
}

/**
 * Set State of switchable Plug
 *
 * @param type $name Name of Plug as defined in fhem
 * @param type $state State; either "on" or "off"
 */
function SetStateSD($name, $state) {
    $syscall = GenerateCmdString("set " . $name . " " . $state);
    exec($syscall);
}

/**
 * Set State of Lamp
 *
 * @param type $name Name of Lamp as defined in fhem
 * @param type $state State; either "on" or "off"
 */
function SetStateLA($name, $state) {
    $syscall = GenerateCmdString("set " . $name . " " . $state);
    exec($syscall);
}

/**
 * Change Setting of Radiator Head
 *
 * @param type $name Name of Head as defined in fhem
 * @param type $state Setpoint; 0..100%
 */
function SetStateHT($name, $state) {
    $syscall = GenerateCmdString("set " . $name . " valve " . $state);
    exec($syscall);
}

/**
 * Set desired Temperature of Room Thermostate
 *
 * @param type $name Name of Thermostate as defined in fhem
 * @param type $state Desired Temperature; any Value und Degree Centigrade
 */
function SetStateFHT($name, $state) {
    $syscall = GenerateCmdString("set " . $name . " desired-temp " . $state);
    exec($syscall);
}

/**
 * Set desired Setpoint of PID Controller
 *
 * @param type $name Name of PID Controller as defined in fhem
 * @param type $state Desired Setpoint; any Value that fits the PID definition
 */
function SetStatePID($name, $state) {
    $syscall = GenerateCmdString("set " . $name . " desired " . $state);
    exec($syscall);
}

/**
 * Get State of Switchable Plug or Lamp
 *
 * @global type $states
 * @param type $name Name of Device as defined in fhem
 * @return type Current State of $name
 */
function GetStateSDLA($name) {
    global $states;
    $result = $states[$name]["state"];
    $res = trim($result);
    return $res;
}

/**
 * Get State of Weather Sensor
 *
 * @global type $states
 * @param type $name Name of Sensor as defined in fhem
 * @return type String containing Temperature and Humidity separated by a single Whitespace
 */
function GetStateTF($name) {
    global $states;
    $result = $states[$name]["temperature"] . " " . $states[$name]["humidity"];
    $res = trim($result);
    return $res;
}

/**
 * Get State of Radiator Head
 *
 * @global type $states
 * @param type $name Name of Head as defined in fhem
 * @return string Current Setpoint of Head; "0" .. "100"
 */
function GetStateHT($name) {
    global $states;
    $result = split(" ", $states[$name]["state"]);
    $res = trim($result[0]);
    if ($res == "")
        $res = "0";
    return $res;
}

/**
 * Get State of Room Thermostate
 *
 * @global type $states
 * @param type $name Name of Room Thermostate as defined in fhem
 * @return string Measured Value of the room Temperature in Degree Centigrade
 */
function GetStateFHT($name) {
    global $states;
    $result = $states[$name]["measured-temp"];
    $resarr = explode(" ", $result);
    $res = "S:" . trim($states[$name]["desired-temp"]) . chr(176) . "C\nI:" . trim($resarr[0]) . chr(176) . "C";
    if ($res == "")
        $res = "0";
    return $res;
}

/**
 * Get State of PID Controller
 *
 * @global type $states
 * @param type $name Name of PID Controller as defined in fhem
 * @return type Current Setpoint and current Value as formatted String
 */
function GetStatePID($name) {
    global $states;
    $result = split(" ", $states[$name]["state"]);
    $res = trim("S:" . $states[$name]["desired"] . chr(176) . "C\n" . "I:" . $result[0] . chr(176) . "C");
    return $res;
}

